"""Load sub-commands for nbcli."""


from nbcli.commands.base import BaseSubCommand      # noqa: F401
from nbcli.commands.init import InitSubCommand      # noqa: F401
from nbcli.commands.models import ModelsSubCommand  # noqa: F401
from nbcli.commands.search import SearchSubCommand  # noqa: F401
from nbcli.commands.filter import FilterSubCommand  # noqa: F401
from nbcli.commands.create import CreateSubCommand  # noqa: F401
from nbcli.commands.shell import ShellSubCommand    # noqa: F401
